# picamera2_example_1.py
# uses the picamera2 module
# 01.03.2023

import sys
import os
import json
import pprint
import time
from argparse import ArgumentParser

import numpy as np
import cv2

from picamera2Utils import initCamera
from picamera2 import Picamera2

# parsing command line args 
parser = ArgumentParser()
parser.add_argument("cameraConfigJS", help="json file to initalise camera module")
args = parser.parse_args()

# process configuration files into dictionaries
try:
    with open(args.cameraConfigJS, 'r') as fid:
        cameraConfig_D = json.load(fid)
except:
    sys.exit("failed to process configuration file")

# file names for storing images
numpy_file = cameraConfig_D['numpy_file']
numpy_file_to_jpg = cameraConfig_D['numpy_file_to_jpg']
jpg_file = cameraConfig_D['jpg_file']

# set verbosity
verbose = True

# get an initialised camera object
picam2 = initCamera(cameraConfig_D, verbose=verbose)
print(f"picam2 : {picam2}")

# querying some camera properties
pp = pprint.PrettyPrinter(indent=4)
print("\ncamera_controls")
pp.pprint(picam2.camera_controls)

# capturing image as array and storing it to file (no image compression)
t1 = time.perf_counter()
img_numpy = picam2.capture_array("main")
t2 = time.perf_counter()
t_elapsed_array_capture = t2 - t1
np.save(numpy_file, img_numpy)
t3 = time.perf_counter()
t_elapsed_array_to_file = t3 - t2
t_elapsed_array = t3 - t1
print(f"\ncaptured image as numpy array with shape: {img_numpy.shape}")
print(f"in file : {numpy_file} ; size (bytes): {os.path.getsize(numpy_file)}\n")
print(f"time to capture array         : {t_elapsed_array_capture:8.3f} seconds")
print(f"time array -> file            : {t_elapsed_array_to_file:8.3f} seconds")
print(f"time capture and store to file: {t_elapsed_array:8.3f} seconds")

# capturing image as array and storing it to file as jpg
# to compress the image image processing library OpenCv is used
t1 = time.perf_counter()
img_numpy = picam2.capture_array("main")
# reordering color channels because opencv assumes BGR order 
# instead of RGB
cv2.imwrite(numpy_file_to_jpg, img_numpy[:,:,[2, 1, 0]])
t2 = time.perf_counter()
t_elapsed_array_to_jpg = t2 - t1
print(f"\ncaptured image as numpy array with shape: {img_numpy.shape}")
print(f"in file : {numpy_file_to_jpg} ; size (bytes): {os.path.getsize(numpy_file_to_jpg)}\n")
print(f"time capture and store to jpg-file: {t_elapsed_array_to_jpg:8.3f} seconds")

# capturing image directly to file as jpeg
t1 = time.perf_counter()
picam2.capture_file(jpg_file)
t2 = time.perf_counter()
t_elapsed_jpeg = t2 - t1
print(f"""\ntime capture jpeg and store to file: {t_elapsed_jpeg:8.3f} seconds
size (bytes): {os.path.getsize(jpg_file)}""")

# getting metadata info
metadata = picam2.capture_metadata()
print("\nmetadata")
pp.pprint(metadata)
  